/*

File: config.c
Author: Neil Cafferkey
Copyright (C) 1999-2001 Neil Cafferkey

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.

*/

#include "config_protos.h"
#include "memory_protos.h"
#include "string_protos.h"
#include "string_set_protos.h"
#include <stdio.h>


String FindHeading(Config config,String heading);


/* Function: ReadConfig
 * ====================
 * Makes a config from the contents of a file.
 */

Config ReadConfig(TEXT *file_name)
{
   return ReadString(file_name);
}


/* Function: GetStringConfigOption
 * ===============================
 */

TEXT *GetStringConfigOption(Config config,TEXT *heading)
{
   TEXT *substring=FindHeading(config,heading);

   substring=(String)strchr(substring,'\t')+1;

   return GetLine(&substring);
}


/* Function: GetNumericConfigOption
 * ================================
 */

ULONG GetNumericConfigOption(Config config,TEXT *heading)
{
   TEXT *substring=FindHeading(config,heading);

   substring=(String)strchr(substring,'\t')+1;

   return StringToULong(GetLine(&substring));
}


/* Function: GetStringSetConfigOption
 * ==================================
 */

StringSet GetStringSetConfigOption(Config config,TEXT *heading)
{
   StringSet string_set=CreateSet();
   TEXT *substring=FindHeading(config,heading);

   substring=(String)strchr(substring,'\n')+1;

   while(*substring=='\t')
   {
      substring++;
      PutInSet(string_set,GetLine(&substring));
   }

   return string_set;
}


/* Function: KillConfig
 * ====================
 */

VOID KillConfig(Config config)
{
   KillString(config);

   return;
}


String FindHeading(Config config,String heading)
{
   String substring=(String)strstr(config,heading);

   if(substring==NULL)
   {
      fprintf(stderr,"Error: couldn't find required heading '%s' in config. file\n",heading);
      NoMoreMem();
      exit(20);
   }

   return substring;
}


