/*

File: instruction_protos.h
Author: Neil Cafferkey
Copyright (C) 1999-2001 Neil Cafferkey

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.

*/

#ifndef _instruction_protos_h
#define _instruction_protos_h


#include "instruction.h"
#include "exec/types.h"
#include "bank.h"
#include "instruction_set.h"
#include <stdio.h>
#include "reverse_graph.h"
#include "source_program.h"


/* Function: CreateInstruction
 * ===========================
 * Creates an Instruction. Possibly private.
 */

Instruction CreateInstruction(Sequence prefix,TEXT *disassembly,
   TEXT *translation,LONGBITS critical_flags,LONGBITS possible_flags,
   LONGBITS definite_flags,UBYTE length,ULONG instruction_type);


/* Function: ReadInstruction
 * =========================
 */

Instruction ReadInstruction(TEXT *string);


/* Function: GetInstructionPrefix
 * ==============================
 */

Sequence GetInstructionPrefix(Instruction instruction);


/* Function: GetInstructionLength
 * ==============================
 */

UBYTE GetInstructionLength(Instruction instruction);


/* Function: GetInstructionType
 * ============================
 */

ULONG GetInstructionType(Instruction instruction);


/* Function: GetDisassemblyTemplate
 * ================================
 */

TEXT *GetDisassemblyTemplate(Instruction instruction);


/* Function: GetTranslationTemplate
 * ================================
 */

TEXT *GetTranslationTemplate(Instruction instruction);


/* Function: GetCriticalFlags
 * ==========================
 */

LONGBITS GetCriticalFlags(Instruction instruction);


/* Function: GetModifiedFlags
 * ==========================
 */

LONGBITS GetModifiedFlags(Instruction instruction);


/* Function: KillInstruction
 * =========================
 */

VOID KillInstruction(Instruction instruction);


/* Function: ShowInstruction
 * =========================
 */

VOID ShowInstruction(Instruction instruction);


#endif
