/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define add_p(s, d, c)  (saturn_hexmode ? add_p_hex (s, d, c) : add_p_bcd (s, d, c))
#define add_wp(s, d) (saturn_hexmode ? add_wp_hex (s, d) : add_wp_bcd (s, d))
#define add_x(s, d)  (saturn_hexmode ? add_x_hex (s, d) : add_x_bcd (s, d))
#define add_m(s, d)  (saturn_hexmode ? add_m_hex (s, d) : add_m_bcd (s, d))
#define add_b(s, d)  (saturn_hexmode ? add_b_hex (s, d) : add_b_bcd (s, d))
#define add_w(s, d)  (saturn_hexmode ? add_w_hex (s, d) : add_w_bcd (s, d))
#define add_a(s, d)  (saturn_hexmode ? add_a_hex (s, d) : add_a_bcd (s, d))


/************************************************************/
static inline void add_p_hex(unsigned char *r2,unsigned char *r1,unsigned char i)
{
r1[i]+=r2[i];
saturn_CARRY=hex_carry[r1[i]];
r1[i]&=0xf;
}

/************************************************************/
static inline void add_p_bcd(unsigned char *r2,unsigned char *r1,unsigned char i)
{
unsigned char tmp;

tmp=r1[i] + r2[i];
saturn_CARRY=add_bcd_carry[tmp];
r1[i]=add_bcd_nibble[tmp];
}

/************************************************************/
static inline void add_wp_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	r1[i]+=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void add_wp_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void add_x_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	r1[i]+=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void add_x_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void add_m_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	r1[i]+=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void add_m_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void add_b_hex(unsigned char *r2,unsigned char *r1)
{
*r1+=*r2;
r1[1]+=r2[1]+hex_carry[*r1];
saturn_CARRY=hex_carry[r1[1]];
*r1&=0xf;
r1[1]&=0xf;
}

/************************************************************/
static inline void add_b_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void add_w_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	r1[i]+=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void add_w_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void add_a_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	r1[i]+=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void add_a_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	tmp=r1[i] + r2[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r1[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
