/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define dec_p(d, c)  (saturn_hexmode ? dec_p_hex (d, c) : dec_p_bcd (d, c))
#define dec_wp(d) (saturn_hexmode ? dec_wp_hex (d) : dec_wp_bcd (d))
#define dec_x(d)  (saturn_hexmode ? dec_x_hex (d) : dec_x_bcd (d))
#define dec_m(d)  (saturn_hexmode ? dec_m_hex (d) : dec_m_bcd (d))
#define dec_b(d)  (saturn_hexmode ? dec_b_hex (d) : dec_b_bcd (d))
#define dec_w(d)  (saturn_hexmode ? dec_w_hex (d) : dec_w_bcd (d))
#define dec_a(d)  (saturn_hexmode ? dec_a_hex (d) : dec_a_bcd (d))

static unsigned char dec_nibble[16] = {15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14};
static unsigned char dec_carry[16] = {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

/************************************************************/
static inline void dec_p_hex(unsigned char *r,unsigned char i)
{
saturn_CARRY=dec_carry[r[i]];
r[i]=dec_nibble[r[i]];
}

/************************************************************/
static inline void dec_p_bcd(unsigned char *r,unsigned char i)
{
unsigned char tmp;

tmp=16 + r[i] - 1;
saturn_CARRY=sub_bcd_carry[tmp];
r[i]=sub_bcd_nibble[tmp];
}

/************************************************************/
static inline void dec_wp_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i <= saturn_P; i++) {
	r[i]-=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_wp_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i <= saturn_P; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_x_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 3; i++) {
	r[i]-=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_x_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 3; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_m_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 3; i < 15; i++) {
	r[i]-=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_m_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 3; i < 15; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_b_hex(unsigned char *r)
{
r[1]-=dec_carry[*r];
*r=dec_nibble[*r];
saturn_CARRY=hex_carry[r[1]];
r[1]&=0xf;
}

/************************************************************/
static inline void dec_b_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 2; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_w_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 16; i++) {
	r[i]-=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_w_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 16; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_a_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 5; i++) {
	r[i]-=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void dec_a_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 5; i++) {
	tmp=16 + r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
