/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define inc_p(d, c)  (saturn_hexmode ? inc_p_hex (d, c) : inc_p_bcd (d, c))
#define inc_wp(d) (saturn_hexmode ? inc_wp_hex (d) : inc_wp_bcd (d))
#define inc_x(d)  (saturn_hexmode ? inc_x_hex (d) : inc_x_bcd (d))
#define inc_m(d)  (saturn_hexmode ? inc_m_hex (d) : inc_m_bcd (d))
#define inc_b(d)  (saturn_hexmode ? inc_b_hex (d) : inc_b_bcd (d))
#define inc_w(d)  (saturn_hexmode ? inc_w_hex (d) : inc_w_bcd (d))
#define inc_a(d)  (saturn_hexmode ? inc_a_hex (d) : inc_a_bcd (d))

static unsigned char inc_nibble[16] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0};
static unsigned char inc_carry[16] = {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

/************************************************************/
static inline void inc_p_hex(unsigned char *r,unsigned char i)
{
r[i]=inc_nibble[r[i]];
saturn_CARRY=inc_carry[r[i]];
}

/************************************************************/
static inline void inc_p_bcd(unsigned char *r,unsigned char i)
{
r[i]++;
saturn_CARRY=add_bcd_carry[r[i]];
r[i]=add_bcd_nibble[r[i]];
}

/************************************************************/
static inline void inc_wp_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i <= saturn_P; i++) {
	r[i]+=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_wp_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i <= saturn_P; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_x_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 3; i++) {
	r[i]+=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_x_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 3; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_m_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 3; i < 15; i++) {
	r[i]+=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_m_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 3; i < 15; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_b_hex(unsigned char *r)
{
*r=inc_nibble[*r];
r[1]+=inc_carry[*r];
saturn_CARRY=hex_carry[r[1]];
r[1]&=0xf;
}

/************************************************************/
static inline void inc_b_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 2; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_w_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 16; i++) {
	r[i]+=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_w_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 16; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_a_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 5; i++) {
	r[i]+=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
static inline void inc_a_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=1;
for (i = 0; i < 5; i++) {
	tmp=r[i] + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	if (!saturn_CARRY) return;
	}
}

/************************************************************/
