/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define lsl_1_p(d, c)  (saturn_hexmode ? lsl_1_p_hex (d, c) : lsl_1_p_bcd (d, c))
#define lsl_1_wp(d) (saturn_hexmode ? lsl_1_wp_hex (d) : lsl_1_wp_bcd (d))
#define lsl_1_x(d)  (saturn_hexmode ? lsl_1_x_hex (d) : lsl_1_x_bcd (d))
#define lsl_1_m(d)  (saturn_hexmode ? lsl_1_m_hex (d) : lsl_1_m_bcd (d))
#define lsl_1_b(d)  (saturn_hexmode ? lsl_1_b_hex (d) : lsl_1_b_bcd (d))
#define lsl_1_w(d)  (saturn_hexmode ? lsl_1_w_hex (d) : lsl_1_w_bcd (d))
#define lsl_1_a(d)  (saturn_hexmode ? lsl_1_a_hex (d) : lsl_1_a_bcd (d))


/************************************************************/
static inline void lsl_1_p_hex(unsigned char *r,unsigned char i)
{
r[i]<<=1;
saturn_CARRY=hex_carry[r[i]];
r[i]&=0xf;
}

/************************************************************/
static inline void lsl_1_p_bcd(unsigned char *r,unsigned char i)
{
r[i]<<=1;
saturn_CARRY=add_bcd_carry[r[i]];
r[i]=add_bcd_nibble[r[i]];
}

/************************************************************/
static inline void lsl_1_wp_hex(unsigned char *r)
{
int i;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	r[i]<<=1;
	r[i]|=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void lsl_1_wp_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void lsl_1_x_hex(unsigned char *r)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	r[i]<<=1;
	r[i]|=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void lsl_1_x_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void lsl_1_m_hex(unsigned char *r)
{
int i;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	r[i]<<=1;
	r[i]|=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void lsl_1_m_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void lsl_1_b_hex(unsigned char *r)
{
*r<<=1;
r[1]<<=1;
r[1]|=*r>>4;
saturn_CARRY=hex_carry[r[1]];
*r&=0xf;
r[1]&=0xf;
}

/************************************************************/
static inline void lsl_1_b_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void lsl_1_w_hex(unsigned char *r)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	r[i]<<=1;
	r[i]|=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void lsl_1_w_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void lsl_1_a_hex(unsigned char *r)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	r[i]<<=1;
	r[i]|=saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void lsl_1_a_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	tmp=(r[i]<<1) + saturn_CARRY;
	saturn_CARRY=add_bcd_carry[tmp];
	r[i]=add_bcd_nibble[tmp];
	}
}

/************************************************************/
