/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define neg_p(d, c)  (saturn_hexmode ? neg_p_hex (d, c) : neg_p_bcd (d, c))
#define neg_wp(d) (saturn_hexmode ? neg_wp_hex (d) : neg_wp_bcd (d))
#define neg_x(d)  (saturn_hexmode ? neg_x_hex (d) : neg_x_bcd (d))
#define neg_m(d)  (saturn_hexmode ? neg_m_hex (d) : neg_m_bcd (d))
#define neg_b(d)  (saturn_hexmode ? neg_b_hex (d) : neg_b_bcd (d))
#define neg_w(d)  (saturn_hexmode ? neg_w_hex (d) : neg_w_bcd (d))
#define neg_a(d)  (saturn_hexmode ? neg_a_hex (d) : neg_a_bcd (d))

static unsigned char neg_nibble[16] = {0,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1};
static unsigned char neg_carry[16] = {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
static unsigned char neg_nc[16] = {0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};

/************************************************************/
static inline void neg_p_hex(unsigned char *r,unsigned char i)
{
saturn_CARRY=neg_carry[r[i]];
r[i]=neg_nibble[r[i]];
}

/************************************************************/
static inline void neg_p_bcd(unsigned char *r,unsigned char i)
{
unsigned char tmp;

tmp=16 - r[i];
saturn_CARRY=sub_bcd_carry[tmp];
r[i]=sub_bcd_nibble[tmp];
}

/************************************************************/
static inline void neg_wp_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i <= saturn_P; i++) {
	r[i]=15 - r[i] + saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void neg_wp_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void neg_x_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 3; i++) {
	r[i]=15 - r[i] + saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void neg_x_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void neg_m_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 3; i < 15; i++) {
	r[i]=15 - r[i] + saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void neg_m_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void neg_b_hex(unsigned char *r)
{
r[1]+=neg_nc[*r];
*r=neg_nibble[*r];
saturn_CARRY=neg_carry[r[1]];
r[1]=neg_nibble[r[1]];
}

/************************************************************/
static inline void neg_b_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void neg_w_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 16; i++) {
	r[i]=15 - r[i] + saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void neg_w_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void neg_a_hex(unsigned char *r)
{
int i;

saturn_CARRY=1;
for (i = 0; i < 5; i++) {
	r[i]=15 - r[i] + saturn_CARRY;
	saturn_CARRY=hex_carry[r[i]];
	r[i]&=0xf;
	}
}

/************************************************************/
static inline void neg_a_bcd(unsigned char *r)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	tmp=16 - r[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
