/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define sub_p(s, d, c)  (saturn_hexmode ? sub_p_hex (s, d, c) : sub_p_bcd (s, d, c))
#define sub_wp(s, d) (saturn_hexmode ? sub_wp_hex (s, d) : sub_wp_bcd (s, d))
#define sub_x(s, d)  (saturn_hexmode ? sub_x_hex (s, d) : sub_x_bcd (s, d))
#define sub_m(s, d)  (saturn_hexmode ? sub_m_hex (s, d) : sub_m_bcd (s, d))
#define sub_b(s, d)  (saturn_hexmode ? sub_b_hex (s, d) : sub_b_bcd (s, d))
#define sub_w(s, d)  (saturn_hexmode ? sub_w_hex (s, d) : sub_w_bcd (s, d))
#define sub_a(s, d)  (saturn_hexmode ? sub_a_hex (s, d) : sub_a_bcd (s, d))


/************************************************************/
static inline void sub_p_hex(unsigned char *r2,unsigned char *r1,unsigned char i)
{
r1[i]-=r2[i];
saturn_CARRY=hex_carry[r1[i]];
r1[i]&=0xf;
}

/************************************************************/
static inline void sub_p_bcd(unsigned char *r2,unsigned char *r1,unsigned char i)
{
unsigned char tmp;

tmp=16 + r1[i] - r2[i];
saturn_CARRY=sub_bcd_carry[tmp];
r1[i]=sub_bcd_nibble[tmp];
}

/************************************************************/
static inline void sub_wp_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_wp_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void sub_x_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_x_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void sub_m_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_m_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void sub_b_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_b_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void sub_w_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_w_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void sub_a_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	r1[i]-=r2[i]+saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void sub_a_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	tmp=16 + r1[i] - r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
