/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#define subr_p(s, d, c)  (saturn_hexmode ? subr_p_hex (s, d, c) : subr_p_bcd (s, d, c))
#define subr_wp(s, d) (saturn_hexmode ? subr_wp_hex (s, d) : subr_wp_bcd (s, d))
#define subr_x(s, d)  (saturn_hexmode ? subr_x_hex (s, d) : subr_x_bcd (s, d))
#define subr_m(s, d)  (saturn_hexmode ? subr_m_hex (s, d) : subr_m_bcd (s, d))
#define subr_b(s, d)  (saturn_hexmode ? subr_b_hex (s, d) : subr_b_bcd (s, d))
#define subr_w(s, d)  (saturn_hexmode ? subr_w_hex (s, d) : subr_w_bcd (s, d))
#define subr_a(s, d)  (saturn_hexmode ? subr_a_hex (s, d) : subr_a_bcd (s, d))


/************************************************************/
static inline void subr_p_hex(unsigned char *r2,unsigned char *r1,unsigned char i)
{
r1[i]=r2[i]-r1[i];
saturn_CARRY=hex_carry[r1[i]];
r1[i]&=0xf;
}

/************************************************************/
static inline void subr_p_bcd(unsigned char *r2,unsigned char *r1,unsigned char i)
{
unsigned char tmp;

tmp=16 - r1[i] + r2[i];
saturn_CARRY=sub_bcd_carry[tmp];
r1[i]=sub_bcd_nibble[tmp];
}

/************************************************************/
static inline void subr_wp_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	r1[i]=r2[i]-r1[i]-saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_wp_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i <= saturn_P; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void subr_x_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	r1[i]=r2[i] - r1[i] - saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_x_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 3; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void subr_m_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	r1[i]=r2[i] - r1[i] - saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_m_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 3; i < 15; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void subr_b_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	r1[i]=r2[i] - r1[i] - saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_b_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 2; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void subr_w_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	r1[i]=r2[i] - r1[i] - saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_w_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 16; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
static inline void subr_a_hex(unsigned char *r2,unsigned char *r1)
{
int i;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	r1[i]=r2[i] - r1[i] - saturn_CARRY;
	saturn_CARRY=hex_carry[r1[i]];
	r1[i]&=0xf;
	}
}

/************************************************************/
static inline void subr_a_bcd(unsigned char *r2,unsigned char *r1)
{
int i;
unsigned char tmp;

saturn_CARRY=0;
for (i = 0; i < 5; i++) {
	tmp=16 - r1[i] + r2[i] - saturn_CARRY;
	saturn_CARRY=sub_bcd_carry[tmp];
	r1[i]=sub_bcd_nibble[tmp];
	}
}

/************************************************************/
