/***************************************************************
 *  Alcuin: A HP48 emulator for Amiga Computers
 * 
 *  (C) 1995 Stephan Schupfer (schupfer@sbox.tu-graz.ac.at)
 *
 */

#include "functions/add.c"
#include "functions/sub.c"
#include "functions/subr.c"
#include "functions/neg.c"
#include "functions/not.c"
#include "functions/inc.c"
#include "functions/dec.c"
#include "functions/lsl.c"
#include "functions/lsl_1.c"
#include "functions/lsr.c"
#include "functions/lsr_1.c"
#include "functions/clr.c"
#include "functions/and.c"
#include "functions/or.c"
#include "functions/add_con.c"
#include "functions/sub_con.c"
#include "functions/load.c"
#include "functions/store.c"
#include "functions/move.c"
#include "functions/exg.c"
#include "functions/eqz.c"
#include "functions/nez.c"
#include "functions/eq.c"
#include "functions/ne.c"
#include "functions/le.c"
#include "functions/lt.c"
#include "functions/ge.c"
#include "functions/gt.c"

static unsigned long inv_nibble_masks [] =
  {
    0xfffffff0,
    0xffffff0f,
    0xfffff0ff,
    0xffff0fff,
    0xfff0ffff,
    0xff0fffff,
    0xf0ffffff,
    0x0fffffff,
    0xfffffff0,
    0xffffff0f,
    0xfffff0ff,
    0xffff0fff,
    0xfff0ffff,
    0xff0fffff,
    0xf0ffffff,
    0x0fffffff,
  };

/************************************************************/
static inline void exg_dp_4(unsigned char *r, word_20 *d)
{
int t,i,k=0;

for (i = 0; i < 4; i++) {
	t = r[i];
	r[i] = (*d >> k) & 0xf;
	*d &= inv_nibble_masks[i];
	*d |= t << k;
	k+=4;
	}
}

/************************************************************/
static inline void exg_dp_5(unsigned char *r, word_20 *d)
{
int t,i,k=0;

for (i = 0; i < 5; i++) {
	t = r[i];
	r[i] = (*d >> k) & 0xf;
	*d &= inv_nibble_masks[i];
	*d |= t << k;
	k+=4;
	}
}

/************************************************************/
static inline void rol_reg(unsigned char *r)
{
int i=16,t;

t = r[15];
while (--i)
	r[i] = r[i-1];
r[0] = t;
}

/************************************************************/
static inline void ror_reg(unsigned char *r)
{
int t,i;

t = r[0];
for (i=0;i<15;i++)
	r[i] = r[i+1];
r[15] = t;
/*if (t) saturn_HST |= SB;*/
}

/************************************************************/
static inline void store_n(word_20 *dat,unsigned char *reg,int n)
{
write_nibble(*dat,reg,n+1);
}

/************************************************************/
static inline void load_n(word_20 *dat, unsigned char *reg, int n)
{
read_nibble_crc(*dat,reg,n+1);
}

/******************************************************************/
static inline void move_dp_4(unsigned char *reg, word_20 *dat)
{
int i, k=0;

for (i = 0; i < 4; i++) {
	*dat &= inv_nibble_masks[i];
	*dat |= reg[i] << k;
	k+=4;
	}
}

/******************************************************************/
static inline void move_dp_5(unsigned char *reg, word_20 *dat)
{
int i, k=0;

for (i = 0; i < 5; i++) {
	*dat &= inv_nibble_masks[i];
	*dat |= reg[i] << k;
	k+=4;
	}
}

/******************************************************************/
static inline void add_con_dp(word_20 *dat, word_20 add)
{
*dat += add;
saturn_CARRY = (*dat & 0xfff00000) ? 1 : 0;
*dat &= 0xfffff;
}

/******************************************************************/
static inline void sub_con_dp(word_20 *dat, word_20 add)
{
*dat -= add;
saturn_CARRY = (*dat & 0xfff00000) ? 1 : 0;
*dat &= 0xfffff;
}

/******************************************************************/
static inline void move_con(unsigned char *dst, unsigned char *src, int n)
{
int i;
unsigned char p=saturn_P;

for (i = 0; i < n; i++) {
	dst[p++] = src[i];
	p&=0xf;
	}
}
/******************************************************************/
static inline void move_con_1(unsigned char *dst, unsigned char src)
{
dst[saturn_P] = src;
}

/******************************************************************/
static inline void move_con_2(unsigned char *dst, unsigned char *src)
{
unsigned char p=saturn_P;

dst[p++] = src[0];
p&=0xf;
dst[p] = src[1];
}

/******************************************************************/
