#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "general.h"

/*

bmp2nokia_icon - Copyright 2001-2002 Michael Kohn (mike@mikekohn.net)
This falls under the Kohnian license.  Please read
it at http://nakentone.naken.cc/

This program is NOT opensourced.  You may not use any part
of this program for your own software.

*/

int main (int argc, char *argv[])
{
FILE *fp;
char filename[1024];
int width,height;
int x,y;
int t,k,r;
int full_sms_size;
int mcc,mnc;
int headers,reverse,screensaver;
char picture[8192];
char message[1024];

  width=72;
  height=14;
  stackptr=0;
  bitptr=7;
  full_sms_size=SMS_SIZE;
  mcc=-1; mnc=-1;
  headers=1;
  reverse=0;
  screensaver=0;
  message[0]=0;

  if (argc<2)
  {
    printf("\nbmp2nokia_icon "VERSION".  Nokia Phone icon maker.\n");
    printf(COPYRIGHT);
    printf("width, height are optional (default 72x14)\n");
    printf("Usage: bmp2nokia_icon [ options ] <input filename>\n");
    printf("       -s <sms size> [ Maximum size of SMS message (default %d)\n",SMS_SIZE);
    printf("       -x [ exclude //SCKL header ]\n");
    printf("       -u [ only user-data-header (8bit) ]\n");
    printf("       -k [ include user-data-header and //SCKL ]\n");
    printf("       -r [ reverse black n white ]\n");
    printf("       -w <width> [ Width of Logo (default %d) ]\n",width);
    printf("       -h <height> [ Height of Logo (default %d) ]\n",height);
    printf("       -l <MCC> <MNC> [ Operator Logo (requires MCC and MNC) ]\n");
    printf("       -m <message> Send a picture message\n");
    printf("       -ss Send as screensaver\n\n");
    exit(0);
  }

  for (t=1; t<argc; t++)
  {
    if (strcmp(argv[t],"-s")==0)
    { full_sms_size=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-w")==0)
    {
      width=atoi(argv[++t]);
      if (width>200) width=200;
    }
      else
    if (strcmp(argv[t],"-x")==0)
    { headers=0; }
      else
    if (strcmp(argv[t],"-k")==0)
    { headers=2; }
      else
    if (strcmp(argv[t],"-u")==0)
    { headers=3; }
      else
    if (strcmp(argv[t],"-r")==0)
    { reverse=1; }
      else
    if (strcmp(argv[t],"-ss")==0)
    {
      screensaver=1;
    }
      else
    if (strcmp(argv[t],"-m")==0)
    {
      strncpy(message,argv[++t],1022);
      message[1022]=0;
    }
      else
    if (strcmp(argv[t],"-h")==0)
    { 
      height=atoi(argv[++t]);
      if (height>200) height=200;
    }
      else
    if (strcmp(argv[t],"-l")==0)
    { 
      mcc=atoi(argv[++t]);
      mnc=atoi(argv[++t]);
    }
      else
    { strncpy(filename,argv[t],1023); }
  }

  printf("\nbmp2nokia_icon "VERSION".  Nokia Phone icon maker.\n");
  printf(COPYRIGHT);

  printf("\nInput Filename: %s\n",filename);
  printf("         Width: %d\n",width);
  printf("        Height: %d\n",height);
  printf("      SMS Size: %d\n",full_sms_size);

  if (mcc!=-1 && mnc!=-1)
  {
    printf("       MCC MNC: %d %d\n",mcc,mnc);
  }

  fp=fopen(filename,"r");
  if (fp==0)
  {
    printf("Could not open file for reading: %s\n",filename);
    exit(0);
  }

  push(0x30,8);   /* Identifier for version */

  if (mcc!=-1 && mnc!=-1)
  {
    t=mcc%10;
    k=(mcc/10)%10;
    r=(mcc/100)%10;
    push((k<<4)+r,8);   /* MCC Mobile Country Code */
    push(0xf0+t,8);     /* MCC Mobile Country Code */
    t=mnc%10;
    k=(mnc/10)%10;
    push((t<<4)+k,8);   /* MNC Mobile Network Code */
    push(0x0A,8);       /* Line feed */
  }
    else
  if (screensaver==1)
  {
    push(0x06,8);
    k=((height*width)/8)+4;
    if (((height*width)%8)!=0) k++;
    push((k>>8),8);
    push((k&255),8);
  }
    else
  if (message[0]!=0)
  {
    push(0x00,8);
    k=strlen(message);
    push((k>>8),8);
    push((k&255),8);
    for(t=0; t<k; t++)
    {
      push(message[t],8);
    }

    push(0x02,8);
    k=((height*width)/8)+4;
    if (((height*width)%8)!=0) k++;
    push((k>>8),8);
    push((k&255),8);
  }

  push(0x00,8);   /* Info Field? */
  push(width,8);   /* height and width DUH */
  push(height,8);
  push(0x01,8);    /* picture depth */

  k=stackptr;

  if (parse_bitmap(fp,width,height,picture)==-1)
  {
    printf("Could not read BMP.\n");
    exit(0);
  }

#ifdef DEBUG
  printf("\n");
#endif

  for (y=height-1; y>=0; y--)
  {
    for (x=0; x<width; x++)
    {
      if (picture[(y*width)+x]==' ')
      {
        push(1^reverse,1);
      }
        else
      {
        push(0^reverse,1);
      }
    }
  }

#ifdef DEBUG
  for (y=height-1; y>=0; y--)
  {
    for (x=0; x<width; x++)
    {
      if (picture[(y*width)+x]==' ')
      {
        printf("*");
      }
        else
      {
        printf("_");
      }
    }
    printf("\n");
  }
#endif

  fclose(fp);

  stackptr++;

  if (mcc!=-1 && mnc!=-1)
  { print_codes(full_sms_size,"1582",headers); }
    else
  if (screensaver==1 || message[0]!=0)
  { print_codes(full_sms_size,"158A",headers); }
    else
  { print_codes(full_sms_size,"1583",headers); }

  return (0);
}



