#include <stdio.h>
#include <stdlib.h>

#include "fileoutput.h"

#define THRESHOLD 50

/* 

bmp_parse.c - Copyright 2001-2002 Michael Kohn (mike@mikekohn.net)
This falls under the Kohnian license.  Please read
it at http://nakentone.naken.cc/

This program is NOT opensourced.  You may not use any part
of this program for your own software.

*/

struct _bitmap_file
{
  unsigned char bfType[2];
  unsigned int bfSize;
  unsigned short int reserved1;
  unsigned short int reserved2;
  unsigned int bfOffs;
};

struct _bitmap_info
{
  unsigned int biSize;
  unsigned int biWidth;
  unsigned int biHeight;
  unsigned short int biPlanes;
  unsigned short int biBitCount;
  unsigned int biCompression;
  unsigned int biSizeImage;
  unsigned int biXPelsPerMetre;
  unsigned int biYPelsPerMetre;
  unsigned int biClrUsed;
  unsigned int biClrImportant;
};

int colors[256];

/*
int read_word(FILE *in)
{
int holding;

  holding=getc(in);
  holding=(getc(in)<<8)+holding;
  return holding;
}

int read_long(FILE *in)
{
int holding;

  holding=getc(in);
  holding=(getc(in)<<8)+holding;
  holding=(getc(in)<<16)+holding;
  holding=(getc(in)<<24)+holding;
  return holding;
}
*/

void read_bitmap_file(FILE *in, struct _bitmap_file *bitmap_file)
{
  bitmap_file->bfType[0]=getc(in);
  bitmap_file->bfType[1]=getc(in);
  bitmap_file->bfSize=read_long(in);
  bitmap_file->reserved1=read_word(in);
  bitmap_file->reserved2=read_word(in);
  bitmap_file->bfOffs=read_long(in);
}

void read_bitmap_info(FILE *in, struct _bitmap_info *bitmap_info)
{
  bitmap_info->biSize=read_long(in);
  bitmap_info->biWidth=read_long(in);
  bitmap_info->biHeight=read_long(in);
  bitmap_info->biPlanes=read_word(in);
  bitmap_info->biBitCount=read_word(in);
  bitmap_info->biCompression=read_long(in);
  bitmap_info->biSizeImage=read_long(in);
  bitmap_info->biXPelsPerMetre=read_long(in);
  bitmap_info->biYPelsPerMetre=read_long(in);
  bitmap_info->biClrUsed=read_long(in);
  bitmap_info->biClrImportant=read_long(in);
}

void raw_uncompressed(FILE *in, int width, int height, int w, int h, char *picture, int bits)
{
int x,y,xout,yout;
int c,ptr,t,r;
int byte_count;

  yout=0;
  ptr=0;

  for (y=0; y<h; y++)
  {
    byte_count=0;
    xout=0;

    for (x=0; x<w; x++)
    {
      if (bits==8 || bits==24)
      {
        if (bits==8) 
        {
          c=getc(in);
          c=colors[c];
          byte_count++;
        }
          else
        if (bits==24)
        {
          c=(getc(in)+getc(in)+getc(in))/3;
          byte_count=byte_count+3;
        }

        if (xout<width)
        {
          if (c<THRESHOLD)
          { picture[ptr++]=' '; }
            else
          { picture[ptr++]='*'; }
          xout++;
        }
      }
        else
      if (bits==4)
      {
        c=getc(in);
        byte_count++;

        if (xout<width)
        {
          if (colors[((c>>4)&15)]<THRESHOLD)
          { picture[ptr++]=' '; }
            else
          { picture[ptr++]='*'; }
          xout++;
        }

        if (xout<width)
        {
          if (colors[(c&15)]<THRESHOLD)
          { picture[ptr++]=' '; }
            else
          { picture[ptr++]='*'; }
          xout++;
        }

        x++;
      }
        else
      if (bits==1)
      {
        c=getc(in);
        byte_count++;

        for (t=7; t>=0; t--)
        {
          if (xout<width)
          {
            if (((c>>t)&1)==0)
            { picture[ptr++]=' '; }
              else
            { picture[ptr++]='*'; }
            xout++;
          }
        }

        x=x+7;
      }
    }

    c=(byte_count%4);
    if (c!=0)
    {
      for (t=c; t<4; t++)
      { getc(in); }
    }

    for (t=xout; t<width; t++) ptr++;
    yout++;
    if (yout>=height) { return; }
  }
}

void raw_compressed8(FILE *in, int width, int height, int w, int h, char *picture, int bits)
{
int x,y,xout,yout;
int c,ptr,t,r;

  yout=0;
  ptr=0;

  for (y=0; y<h; y++)
  {
    xout=0;
    x=0;
    while(x<10000)
    {
      c=getc(in);
      if (c==EOF) return;

      if (c!=0)
      {
        r=getc(in);

        for (t=0; t<c; t++)
        {
          if (xout<width)
          {
            if (colors[r]<THRESHOLD)
            { picture[ptr++]=' '; }
              else
            { picture[ptr++]='*'; }
            xout++;
          }
        }

        x=x+c;
      }
        else
      {
        r=getc(in);

        if (r==0)
        {
          /* if (x!=0) x=1000000; */
          break;
        }
          else
        if (r==1)
        {
          return;
        }
          else
        if (r==2)
        {
          x=getc(in);
          y=getc(in);
          xout=x;
          yout=y;
          return;
        }

        for (t=0; t<r; t++)
        {
          c=getc(in);
          if (xout<width)
          {
            if (colors[c]<THRESHOLD)
            { picture[ptr++]=' '; }
              else
            { picture[ptr++]='*'; }
            xout++;
          }
        }

        x=x+r;
      }
    }

    for (t=xout; t<width; t++) ptr++;

    yout++;
    if (yout>=height) { return; }
  }
}

int parse_bitmap(FILE *in, int width, int height, char *picture)
{
struct _bitmap_file bitmap_file;
struct _bitmap_info bitmap_info;
int t,c;

  read_bitmap_file(in, &bitmap_file);

  if (bitmap_file.bfType[0]!='B' || bitmap_file.bfType[1]!='M')
  {
    printf("Not a bitmap.\n");
    return -1;
  }

  read_bitmap_info(in, &bitmap_info);

#ifdef DEBUG
  printf("Bitmap File Header\n");
  printf("----------------------------------------------\n");
  printf("         bfType: %c%c\n",bitmap_file.bfType[0],bitmap_file.bfType[1]);
  printf("         bfSize: %d\n",bitmap_file.bfSize);
  printf("      reserved1: %d\n",bitmap_file.reserved1);
  printf("      reserved2: %d\n",bitmap_file.reserved2);
  printf("         bfOffs: %d\n",bitmap_file.bfOffs);
  printf("----------------------------------------------\n");
  printf("Bitmap Info Header\n");
  printf("----------------------------------------------\n");
  printf("         biSize: %d\n",bitmap_info.biSize);
  printf("        biWidth: %d\n",bitmap_info.biWidth);
  printf("       biHeight: %d\n",bitmap_info.biHeight);
  printf("       biPlanes: %d\n",bitmap_info.biPlanes);
  printf("     biBitCount: %d\n",bitmap_info.biBitCount);
  printf("  biCompression: %d\n",bitmap_info.biCompression);
  printf("    biSizeImage: %d\n",bitmap_info.biSizeImage);
  printf("biXPelsPerMetre: %d\n",bitmap_info.biXPelsPerMetre);
  printf("biYPelsPerMetre: %d\n",bitmap_info.biYPelsPerMetre);
  printf("      biClrUsed: %d\n",bitmap_info.biClrUsed);
  printf(" biClrImportant: %d\n",bitmap_info.biClrImportant);
  printf("----------------------------------------------\n");
#endif

  for (t=0; t<bitmap_info.biClrImportant; t++)
  {
    c=(getc(in)+getc(in)+getc(in))/3;
    getc(in);
    colors[t]=c;
  }

  c=width*height;
  for (t=0; t<c; t++) picture[t]=' ';

  fseek(in,bitmap_file.bfOffs,0);

  if (bitmap_info.biCompression==0)
  {
    raw_uncompressed(in,width,height,bitmap_info.biWidth,bitmap_info.biHeight,picture,bitmap_info.biBitCount);
  }
    else
  if (bitmap_info.biCompression==1)
  {
    raw_compressed8(in,width,height,bitmap_info.biWidth,bitmap_info.biHeight,picture,bitmap_info.biBitCount);
  }
    else
  if (bitmap_info.biCompression==2)
  {
    printf("RLE4 compression is not supported at this time.\n");
  }
    else
  {
    printf("This type of compression is not supported at this time.\n");
  }

  return 0;
}


