#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "general.h"
#include "fileoutput.h"

/*

ringtonetools - Copyright 2001-2002 Michael Kohn (mike@mikekohn.net)
This falls under the Kohnian license.  Please read                 
it at http://nakentone.naken.cc/                                   

This program is NOT opensourced.  You may not use any part
of this program for your own software.

*/

int get_intype(char *param)
{
  if (strcasecmp(param,"rtttl")==0)
  { return 0; }
    else
  if (strcasecmp(param,"rtx")==0)
  { return -2; }
    else
  if (strcasecmp(param,"imelody")==0)
  { return 1; }
    else
  if (strcasecmp(param,"wav")==0)
  { return 2; }
    else
  if (strcasecmp(param,"bmp")==0)
  { return 3; }
    else
  if (strcasecmp(param,"text")==0)
  { return 4; }
    else
  if (strcasecmp(param,"midi")==0 || strcmp(param,"mid")==0)
  { return 5; }
    else
  if (strcasecmp(param,"kws")==0 || strcmp(param,"mid")==0)
  { return 6; }

  return -1;
}

int get_outtype(char *param)
{
  if (strcasecmp(param,"wav")==0)
  { return 0; }
    else
  if (strcasecmp(param,"kws")==0)
  { return 1; }
    else
  if (strcasecmp(param,"mot")==0)
  { return 2; }
    else
  if (strcasecmp(param,"pdb")==0)
  { return 3; }
    else
  if (strcasecmp(param,"nok")==0 || strcmp(param,"nokia")==0)
  { return 4; }
    else
  if (strcasecmp(param,"rtttl")==0)
  { return 5; }
    else
  if (strcasecmp(param,"samsung1")==0)
  { return 6; }
    else
  if (strcasecmp(param,"samsung2")==0)
  { return 7; }

  return -1;
}

int main (int argc, char *argv[])
{
char filename[1024];
char outname[1024];
char message[1024];
int t;
struct note_t note;
int file_type,in_type;

  memset(&note, 0, sizeof(struct note_t));

  note.sample_rate=44100;
  note.bytes=2;
  note.bpm=140;
  note.a440=5;
  note.ats=1;

  note.width=72;
  note.height=14;
  note.mcc=-1;
  note.mnc=-1;

  note.full_sms_size=SMS_SIZE;
  note.headers=1;

  file_type=-1;
  in_type=-1;

  outname[0]=0;
  filename[0]=0;
  message[0]=0;

  if (argc<2)
  {
    printf("\nringtonetools "VERSION" - Convert and create ringtones/logos for mobile phones.\n");
    printf(COPYRIGHT);
    printf("Usage: ringtonetools [ options ] <input filename>\n");
    printf("       -o <output filename>\n");
    printf("       -intype <filename type> [ imelody/rtttl/rtx/midi/kws/bmp/text ]\n");
    printf("       -outtype <output type> [ wav/kws/mot/pdb/nokia/rtttl/samsung1/samsung2 ]\n");
    printf("       -name <song name>\n");
    printf("   iMelody options:\n");
    printf("       -a [force ringtonetools to ignore @'s in imelody's]\n");
    printf("       -t <tempo [default: %d]>\n",note.bpm);
    printf("   wav output options:\n");
    printf("       -b <bits 8 or 16 for wav only [default: 16]>\n");
    printf("       -f <sampling frequency for wav only [default: 44100]>\n");
    printf("   logo options:\n");
    printf("       -h <height> [ Height of Logo (default %d) ]\n",note.height);
    printf("       -w <width> [ Width of Logo (default %d) ]\n",note.width);
    printf("       -l <MCC> <MNC> [ Operator Logo (requires MCC and MNC) ]\n");
    printf("       -m <message> Send a picture message\n");
    printf("       -r [ reverse black n white ]\n");
    printf("       -ss Send as screensaver\n");
    printf("   nokia options:\n");
    printf("       -x [ exclude //SCKL header ]\n");
    printf("       -u [ only user-data-header (8bit) ]\n");
    printf("       -k [ include user-data-header and //SCKL ]\n");
    printf("       -s <sms size> [ Maximum size of SMS message (default %d)\n",SMS_SIZE);
    printf("       -transpose <pos or neg num > [ raise or lower song by n octaves ]\n");
    printf("   midi options:\n");
    printf("       -channel [ channel ]\n");
    printf("       -track [ track ]\n");

    exit(0);
  }

  for (t=1; t<argc; t++)
  {
    if (strcmp(argv[t],"-a")==0)
    { note.ats=0; }
      else
    if (strcmp(argv[t],"-t")==0)
    { note.bpm=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-f")==0)
    { note.sample_rate=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-r")==0)
    { note.bmp_flags^=1; }
      else
    if (strcmp(argv[t],"-h")==0)
    {
      note.height=atoi(argv[++t]);
      if (note.height>200) note.height=200;
    }
      else
    if (strcmp(argv[t],"-w")==0)
    {
      note.width=atoi(argv[++t]);
      if (note.width>200) note.width=200;
    }
      else
    if (strcmp(argv[t],"-x")==0)
    { note.headers=0; }
      else
    if (strcmp(argv[t],"-k")==0)
    { note.headers=2; }
      else
    if (strcmp(argv[t],"-u")==0)
    { note.headers=3; }
      else
    if (strcmp(argv[t],"-ss")==0)
    { 
      note.bmp_flags^=2;
    }
      else
    if (strcmp(argv[t],"-s")==0)
    { note.full_sms_size=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-channel")==0)
    { note.mcc=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-track")==0)
    { note.mnc=atoi(argv[++t]); }
      else
    if (strcmp(argv[t],"-b")==0)
    { 
      note.bytes=atoi(argv[++t]);
      if (note.bytes==16)
      { note.bytes=2; }
        else
      { note.bytes=1; }
    }
      else
    if (strcmp(argv[t],"-l")==0)
    {
      note.mcc=atoi(argv[++t]);
      note.mnc=atoi(argv[++t]);
    }
      else
    if (strcmp(argv[t],"-transpose")==0)
    {
      note.transpose=atoi(argv[++t]);
    }
      else
    if (strcmp(argv[t],"-m")==0)
    { 
      strcpy(message,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-o")==0)
    { 
      strcpy(outname,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-name")==0)
    { 
      strncpy(note.songname,argv[++t],15);
      note.songname[15]=0;
    }
      else
    if (strcmp(argv[t],"-outtype")==0)
    {
      t++;
      file_type=get_outtype(argv[t]);
    }
      else
    if (strcmp(argv[t],"-intype")==0)
    {
      t++;
      in_type=get_intype(argv[t]);

      if (in_type==0) note.a440=5;
        else
      if (in_type==-2)
      {
        note.a440=4;
        in_type=0;
      }
    }
      else
    { strncpy(filename,argv[t],1023); }
  }

  if (in_type==-1)
  {
    t=strlen(filename);
    while(t>0 && filename[t]!='.') t--;

    if (filename[t]=='.')
    {
      t++;
      in_type=get_intype(&filename[t]);
    }

    if (in_type==-1) in_type=0;
  }

  if (file_type==-1 && outname[0]!=0)
  {
    t=strlen(outname);
    while(t>0 && outname[t]!='.') t--;

    if (outname[t]=='.')
    {
      t++;
      file_type=get_outtype(&outname[t]);
    }
  }

  if (file_type==-1) file_type=0;

  if (outname[0]==0)
  {
    strcpy(outname,filename);

    t=strlen(outname);

    while(t>0)
    {
      if (outname[t]=='.')
      {
        outname[t+1]=0;
      }
      t--;
    }

    if (file_type==0)
    { strcat(outname,"wav"); }
      else
    if (file_type==1)
    { strcat(outname,"kws"); }
      else
    if (file_type==2)
    { strcat(outname,"mot"); }
      else
    if (file_type==3)
    { strcat(outname,"pdb"); }
      else
    if (file_type==4)
    { strcat(outname,"nokia"); }
  }

  printf("\nRingtonetools "VERSION"\n");
  printf(COPYRIGHT);

  printf(" Infile: %s (%d)\n",filename,in_type);
  printf("Outfile: %s (%d)\n",outname,file_type);

  if (in_type==0)
  { parse_rtttl(filename,outname,file_type,&note); }
    else
  if (in_type==1)
  { parse_imelody(filename,outname,file_type,&note); }
    else
  if (in_type==2)
  {
    if (file_type==3)
    { wav2pdb(filename,outname,&note); }
      else
    { 
      printf("Wav's can only be converted to pdb's.\n");
      parse_wav(filename,outname,file_type,&note);
    }
  }
    else
  if (in_type==3 || in_type==4)
  {
    if (file_type!=4)
    { printf("Logos can only be done for Nokia phones right now.\n"); }
      else
    { parse_logo(filename,outname,in_type,file_type,&note,message); }
  }
    else
  if (in_type==5)
  { parse_midi(filename,outname,file_type,&note); }
    else
  if (in_type==6)
  { parse_kws(filename,outname,file_type,&note); }

  return (0);
}



