#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fileoutput.h"
#include "nokia.h"

#define SAMPLES 700
#define CUTOFF 7
#define WAVES 2
#define ZEROS 1000

/*

Ringtonetools - Copyright 2001-2002 Michael Kohn (mike@mikekohn.net)
This falls under the Kohnian license.  Please read
it at http://nakentone.naken.cc/

This program is NOT opensourced.  You may not use any part
of this program for your own software.

*/

int get_duration(float duration);

/* WAV */

void write_wav_header(FILE *out, struct note_t *note)
{
  /* Write RIFF header */

  fprintf(out,"RIFF");
  write_long(out,4);
  fprintf(out,"WAVE");

  /* Write fmt header */

  fprintf(out,"fmt ");
  write_long(out,16);
  write_word(out,1);
  write_word(out,1);
  write_long(out,note->sample_rate);
  write_long(out,note->sample_rate*note->bytes);
  if (note->bytes==1)
  { write_word(out,1); }
    else
  if (note->bytes==2)
  { write_word(out,2); }
  write_word(out,note->bytes*8);

  /* Write the start of the data header */

  fprintf(out,"data");
  write_long(out,0);
}

void write_wav_note(FILE *out, struct note_t *note)
{
unsigned int samples_per_note;
double samples_per_wave;
double waveform;
double waveform_slope;
double actual_freq;
double waveform_x;
/*              P  C         C#       C        D#       E        F */
double freqs[]={ 0, 261.625, 277.175, 293.675, 311.125, 329.625, 349.225, 
                370, 392, 415.3, 440, 466.15, 493.883 };
/*              F#   G    G#     A    A#      B */
int t;
short int sample16;
char sample8;
int style_length;

  if (note->transpose>0) note->scale+=note->transpose;

  note->samples_per_beat=(double)note->sample_rate*(60/(double)note->bpm);

  samples_per_note=(double)note->samples_per_beat*(double)((double)4/(double)(1<<(note->length)));

  if (note->modifier==1)
  { samples_per_note=samples_per_note+(samples_per_note/2); }
    else
  if (note->modifier==2)
  { samples_per_note=samples_per_note+(samples_per_note/2)+(samples_per_note/4); }

  if (note->tone==0)
  {
    for (t=0; t<samples_per_note; t++)
    {
      if (note->bytes==1)
      { putc(0,out); }
        else
      { putc(0,out); putc(0,out); }
    }
  }
    else
  {
    if (note->tone>12) note->tone=1;
    /* actual_freq=freqs[note->tone]/2; */

    actual_freq=freqs[note->tone];
    actual_freq=actual_freq*(double)(1<<note->scale);

    samples_per_wave=(double)note->sample_rate/(double)actual_freq;
    waveform_x=0;

    if (note->style==0)
    { style_length=samples_per_note-(double)note->sample_rate/64; }
      else
    if (note->style==2)
    { style_length=samples_per_note/3; }
      else
    { style_length=samples_per_note; }

    if (note->bytes==1)
    {
      waveform_slope=255/(double)samples_per_wave;
      waveform=127;

/* printf("slope=%f  samples_per_wave=%d\n",waveform_slope,(int)samples_per_wave); */

      for (t=0; t<samples_per_note; t++)
      {
        if (t<style_length)
        {
          if (waveform_x>samples_per_wave)
          {
            waveform=127;
            waveform_x=waveform_x-samples_per_wave;
          }

          sample8=waveform;
          putc(sample8,out);
          waveform=waveform-waveform_slope;
          if (waveform<-127) waveform=-127;
          waveform_x=waveform_x+1;
        }
          else
        { putc(0,out); }
      }
    }
      else
    if (note->bytes==2)
    {
      waveform_slope=65535/(double)samples_per_wave;
      waveform=32767;

      for (t=0; t<samples_per_note; t++)
      {
        if (t<style_length)
        {
          if (waveform_x>samples_per_wave)
          {
            waveform=65535;
            waveform_x=waveform_x-samples_per_wave;
          }

          sample16=waveform;
          write_word(out,sample16);
          waveform=waveform-waveform_slope;
          if (waveform<-32767) waveform=-32767;
          waveform_x=waveform_x+1;
        }
          else
        { putc(0,out); putc(0,out); }
      }
    } 
  }
}

void write_wav_footer(FILE *out, struct note_t *note)
{
unsigned int data_length,t;

  t=ftell(out);
  data_length=t-note->marker-4;
  fseek(out,note->marker,0);
  write_long(out,data_length);
  fseek(out,4,0);
  write_long(out,t-8);
}

int get_note_from_freq(float freq, struct note_t *note)
{
double freqs[13];
/*                 P  C        C#       C        D#       E        F */
double freqs_t[]={ 0, 261.625, 277.175, 293.675, 311.125, 329.625, 349.225,
                370, 392, 415.3, 440, 466.15, 493.883 };
/*              F#   G    G#     A    A#      B */
int t,m;

  if (freq<freqs_t[1]-5) return 0;

  m=0;
  if (freq>freqs_t[12]+12) m=1;
  if (freq>(freqs_t[12]*2)+24) m=2;

  for(t=0; t<12; t++)
  { freqs[t]=freqs_t[t+1]*(1<<m); }

  if (freq<freqs[0]) freq=freqs[0];

  for (t=0; t<12; t++)
  {
    if (freqs[t]==freq) return (t+(m*12))+1;

    if (freqs[t]<freq && freq<freqs[t+1])
    {
      if (freq-freqs[t]<freqs[t+1]-freq)
      { return (t+(m*12))+1; }
        else
      { return (t+(m*12))+2; }
    }
  }

  return t;
}

int parse_wav(char *filename, char *outname, int file_type, struct note_t *note)
{
FILE *in,*out;
char notes[2048];
char name[5],riff_type[5];
int duration[2048],sample_rate,channels,bytes;
int count,quarter_note,curr_note,last_note;
short int curr_sample;
int waves,samples,zeros;
int peak=1,length,t,a,ch,zero;
int datalength;
double freq;

  in=fopen(filename,"r");
  if (in==0)
  {
    printf("Could not open file for reading: %s\n",filename);
    return -1;
  }

  read_chars(in,name,4);
  t=read_long(in);
  read_chars(in,riff_type,4);

  if (strcasecmp(name,"RIFF")!=0 || strcasecmp(riff_type,"wave")!=0)
  {
    printf("Invalid WAV format.\n");
    return -1;
  }

  read_chars(in,name,4);
  read_long(in);
  read_word(in);
  channels=read_word(in);
  sample_rate=read_long(in);
  read_long(in);
  bytes=read_word(in);
  read_word(in);

  read_chars(in,name,4);
  datalength=read_long(in);

  curr_sample=0;
  samples=0;
  waves=0;
  zeros=0;
  t=0;
  count=0;
  last_note=-1;
  length=0;

#ifdef DEBUG
printf("going to read %d bytes\n",datalength);
printf("bytes_per_sample=%d   channels=%d\n",bytes,channels);
#endif

  while (t<datalength)
  {
    if (bytes==1)
    {
      ch=getc(in);
      curr_sample=ch*65535/255;
    }
      else
    if (bytes==2 && channels==2)
    {
      ch=getc(in)+getc(in);
      ch=ch/2;
      curr_sample=ch*65535/255;
    }
      else
    if (bytes==2 && channels==1)
    {
      curr_sample=read_word(in);
    }
      else
    if (bytes==4 && channels==2)
    {
/*
      ch=(short int)read_word(in)+(short int)read_word(in);
      curr_sample=ch/2;
*/
      read_word(in);
      curr_sample=(short int)read_word(in);
    }

#ifdef DEBUG
/* printf("curr_sample=%d   t=%d\n",curr_sample,t); */
#endif

    t=t+note->bytes;

    if (curr_sample<CUTOFF && curr_sample>-CUTOFF)
    { curr_sample=0; }

    if (curr_sample==0)
    {
      zero++;
      if (zero>=ZEROS)
      {
        if (count!=0)
        {
          notes[count]=0;
          duration[count]=((double)zeros/(double)sample_rate)*1000;
          count++;
        }
        zero=0;
        samples=0;
        waves=0;
      }
    }
      else
    {
      zero=0;

      if (curr_sample<0 && peak==1)
      {
        peak=-1;
      }
        else
      if (curr_sample>0 && peak==-1)
      {
        peak=1;
        waves++;
      }
    }

    samples++;

    /* if (samples>=SAMPLES) */
    if (waves>=WAVES)
    {
      freq=(double)waves/((double)samples/(double)sample_rate);
      curr_note=get_note_from_freq(freq,note);

#ifdef DEBUG
printf("waves: %d  freq: %f    curr_note: %d\n",waves,freq,curr_note);
#endif

      if (curr_note!=last_note)
      {
        if (last_note!=-1)
        {
          if (!(count==0 && last_note==0))
          {
            notes[count]=last_note;
            /* duration[count]=((double)length/(double)SAMPLES)*1000; */
            duration[count]=((double)length/(double)sample_rate)*1000;
#ifdef DEBUG
printf("Note %d  Dur %d  sam %d\n",last_note,duration[count],samples);
#endif
            if (duration[count]>10)
            { count++; }
          }
          length=0;
        }
      }

      length=length+samples;

      last_note=curr_note;
      waves=0;
      samples=0;
    }
  }


  if (count==0)
  {
    printf("No notes were processed.  Aborting!\n");
    return -1;
  }

#ifdef DEBUG
printf("count: %d\n",count);
#endif

  quarter_note=duration[0];

  t=0;
  while(t<8)
  {
    note->bpm=60000/quarter_note;
    if (note->bpm<250) break;

    quarter_note=quarter_note*2;
    t++;
  }

  t=0;
  while(t<8)
  {
    if (note->bpm>100) break;
    quarter_note=quarter_note/2;
    note->bpm=60000/quarter_note;

    t++;
  }

#ifdef DEBUG
printf("Quarter note: %d ms\n",quarter_note);
#endif

  t=0;
  if (note->songname[0]==0)
  {
    while(filename[t]!='.' && filename[t]!=0 && t<14)
    {
      note->songname[t]=filename[t];
      t++;
    }
    note->songname[t]=0;
  }

  out=fopen(outname,"w+");
  if (out==0)
  {
    printf("Could not open file for writing: %s\n",outname);
    fclose(in);
    return -1;
  }

  header_route(out,note,0,file_type);

  for (t=0; t<count; t++)
  {
    if (notes[t]==0)
    {
      note->tone=0;
    }
      else
    {
      note->tone=(notes[t]-1)%12+1;
      note->scale=(notes[t]-1)/12;
    }

    a=get_duration((float)duration[t]/((float)quarter_note*4));
    note->length=a/2;
    note->modifier=(a%2)^1;

    if (note->length>5) continue;

    note_route(out,note,0,file_type);
  }

  footer_route(out,note,0,file_type);

  fclose(in);
  fclose(out);

  return (0);

}

